import 'dart:collection';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';
import 'package:provider/provider.dart';

import '/flutter_flow/flutter_flow_theme.dart';
import '/flutter_flow/flutter_flow_util.dart';
import 'card_item_template_component_model.dart';

export 'card_item_template_component_model.dart';

class CardItemTemplateComponentWidget extends StatefulWidget {
  const CardItemTemplateComponentWidget({
    Key? key,
    required this.labelsHashMap,
    required this.statusHashMap,
    required this.imagePath,
    required this.onTapCardItemAction,
  }) : super(key: key);

  final Map<String, String>? labelsHashMap;
  final List<Map<String, Color>?> statusHashMap;
  final String? imagePath;
  final Future Function()? onTapCardItemAction;

  @override
  State<CardItemTemplateComponentWidget> createState() =>
      _CardItemTemplateComponentWidgetState();
}

class _CardItemTemplateComponentWidgetState
    extends State<CardItemTemplateComponentWidget> {
  late CardItemTemplateComponentModel _model;
  late LinkedHashMap<String, String> labelsLinkedHashMap;
  late List<LinkedHashMap<String, Color>> statusLinkedHashMap;

  @override
  void initState() {
    super.initState();
    _model = createModel(context, () => CardItemTemplateComponentModel());
    labelsLinkedHashMap = LinkedHashMap.from(widget.labelsHashMap ?? {});
    statusLinkedHashMap = widget.statusHashMap
        .map((map) => LinkedHashMap<String, Color>.from(map ?? {}))
        .toList();
  }

  @override
  void dispose() {
    super.dispose();
    _model.maybeDispose();
  }

  String truncate(int cutoff, String text) {
    return (text.length <= cutoff) ? text : '${text.substring(0, cutoff)}...';
  }

  List<Widget> _generateLabels() {
    return labelsLinkedHashMap.entries.map((entry) {
      final key = entry.key;
      final text = entry.value.toString();

      return Container(
        color: FlutterFlowTheme.of(context).primary,
        child: Row(
          children: [
            Text(
              key,
              style: FlutterFlowTheme.of(context).bodyMedium.override(
                    fontFamily: FlutterFlowTheme.of(context).bodyMediumFamily,
                    letterSpacing: 0.0,
                    fontWeight: FontWeight.bold,
                    useGoogleFonts: GoogleFonts.asMap().containsKey(
                        FlutterFlowTheme.of(context).bodyMediumFamily),
                    color: FlutterFlowTheme.of(context).primaryText,
                  ),
            ),
            Expanded(
              child: Text(
                text,
                overflow: TextOverflow.ellipsis,
                style: FlutterFlowTheme.of(context).bodyMedium.override(
                      fontFamily: FlutterFlowTheme.of(context).bodyMediumFamily,
                      fontSize: 12.5,
                      letterSpacing: 0.0,
                      fontWeight: FontWeight.bold,
                      useGoogleFonts: GoogleFonts.asMap().containsKey(
                          FlutterFlowTheme.of(context).bodyMediumFamily),
                    ),
              ),
            )
          ].divide(const Flexible(child: SizedBox(width: 25))),
        ),
      );
    }).toList();
  }

  Widget _generateImage() {
    return Flexible(
      child: SizedBox(
        width: 100,
        height: 100,
        child: ClipRRect(
          borderRadius: BorderRadius.circular(20),
          child: CachedNetworkImage(
            fadeInDuration: const Duration(milliseconds: 500),
            fadeOutDuration: const Duration(milliseconds: 500),
            imageUrl: widget.imagePath ?? '',
            fit: BoxFit.cover,
          ),
        ),
      ),
    );
  }

  List<Widget> _generateStatus() {
    return statusLinkedHashMap.expand((statusLinked) {
      return statusLinked.entries.map((entry) {
        final text = entry.key;
        final color = entry.value;

        return Container(
          padding: const EdgeInsets.all(5),
          decoration: BoxDecoration(
            color: color,
            borderRadius: BorderRadius.circular(5),
          ),
          child: Text(
            text,
            overflow: TextOverflow.ellipsis,
            style: TextStyle(
              color: FlutterFlowTheme.of(context).info,
              fontSize: 12,
              fontWeight: FontWeight.bold,
            ),
          ),
        );
      }).toList();
    }).toList();
  }

  Widget _buildContent() {
    final screenWidth = MediaQuery.of(context).size.width;

    if (screenWidth > 360) {
      return Row(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Expanded(
            child: Container(
              color: FlutterFlowTheme.of(context).error,
              child: Column(
                mainAxisSize: MainAxisSize.max,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  ..._generateLabels(),
                  const SizedBox(height: 5),
                  Wrap(
                    spacing: 8,
                    runSpacing: 8,
                    children: _generateStatus(),
                  ),
                ],
              ),
            ),
          ),
          _generateImage(),
        ],
      );
    } else {
      return Column(
        mainAxisSize: MainAxisSize.max,
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          _generateImage(),
          const SizedBox(height: 5),
          Expanded(
            child: Column(
              mainAxisSize: MainAxisSize.max,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                ..._generateLabels(),
                const SizedBox(height: 5),
                Wrap(
                  spacing: 8,
                  runSpacing: 8,
                  children: _generateStatus(),
                ),
              ],
            ),
          ),
        ],
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    context.watch<AppState>();

    return InkWell(
      splashColor: Colors.transparent,
      focusColor: Colors.transparent,
      hoverColor: Colors.transparent,
      highlightColor: Colors.transparent,
      onTap: () async => await widget.onTapCardItemAction?.call(),
      child: Padding(
        padding: const EdgeInsets.all(8),
        child: Card(
          clipBehavior: Clip.antiAliasWithSaveLayer,
          color: FlutterFlowTheme.of(context).primaryBackground,
          elevation: 5.0,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(8.0),
          ),
          child: _buildContent(),
        ),
      ),
    );
  }
}
